@@ -28,11 +28,10 @@ extern "C" {
   #include <sys/types.h> // for chmod
   #include <sys/stat.h> // for chmod
   #include <glib/gi18n.h>
-  #include <glib/gmessages.h> // for g_assert
 }
 #include <pan/general/debug.h>
-#include <pan/general/foreach.h>
 #include <pan/general/log.h>
+#include <pan/general/macros.h>
 #include <pan/general/messages.h>
 #include <pan/general/quark.h>
 #include <pan/general/time-elapsed.h>
