@@ -34,7 +34,7 @@ extern "C"
 
 #include <pan/general/debug.h>
 #include <pan/general/file-util.h>
-#include <pan/general/foreach.h>
+#include <pan/general/macros.h>
 #include <pan/general/messages.h>
 #include <pan/general/log.h>
 #include <pan/general/string-view.h>
@@ -406,7 +406,8 @@ ArticleCache :: get_message (const mid_sequence_t& mids)
    // load the streams
    typedef std::vector<GMimeStream*> streams_t;
    streams_t streams;
-   const bool in_memory (mids.size() <= 2u);
+   //const bool in_memory (mids.size() <= 2u);
+   const bool in_memory (true); // workaround for bug #439841
    foreach_const (mid_sequence_t, mids, it) {
       const Quark mid (*it);
       GMimeStream * stream (0);
