@@ -25,11 +25,12 @@
 #include <map>
 #include <vector>
 
+#include <pan/general/macros.h>
 #include <pan/general/quark.h>
 #include <pan/general/string-view.h>
+#include <pan/usenet-utils/scorefile.h>
 #include <pan/data/article.h>
 #include <pan/data/article-cache.h>
-#include <pan/data/scorefile.h>
 #include <pan/data/server-info.h>
 
 namespace pan
@@ -89,6 +90,7 @@ namespace pan
     int sig_type;
     std::string signature_file;
     std::string attribution;
+    std::string fqdn;
     Quark posting_server;
 
     void get_from_header (std::string& s) const {
@@ -121,11 +123,6 @@ namespace pan
     public:
       virtual ~Profiles () {}
 
-    public:
-      typedef std::vector<std::string> strings_t;
-      virtual void get_editors (strings_t& setme) const = 0;
-      virtual const std::string& get_active_editor () const = 0;
-
     public:
       virtual std::set<std::string> get_profile_names () const = 0;
       virtual bool has_profiles () const = 0;
@@ -135,8 +132,6 @@ namespace pan
     public:
       virtual void delete_profile (const std::string& profile_name) = 0;
       virtual void add_profile (const std::string& profile_name, const Profile& profile) = 0;
-      virtual void set_editors (const strings_t&) = 0;
-      virtual void set_active_editor (const StringView&) = 0;
       
     protected:
       Profiles () {}
@@ -223,11 +218,12 @@ namespace pan
         virtual ~Listener () {}
 
         virtual void on_grouplist_rebuilt () {}
-        virtual void on_group_read (const Quark& group) {}
-        virtual void on_group_subscribe (const Quark& group, bool sub) {}
-        virtual void on_group_counts (const Quark& group,
-                                      unsigned long unread,
-                                      unsigned long total) {}
+        virtual void on_group_read (const Quark& group UNUSED) {}
+        virtual void on_group_subscribe (const Quark & group UNUSED,
+                                         bool          sub   UNUSED) {}
+        virtual void on_group_counts (const Quark& group   UNUSED,
+                                      unsigned long unread UNUSED,
+                                      unsigned long total  UNUSED) {}
       };
 
       void add_listener (Listener * l);
@@ -267,6 +263,8 @@ namespace pan
         std::string description;
         /** y, n, or m for posting, no posting, moderated */
         char permission;
+
+        NewGroup(): permission('?') {}
       };
 
       /**
@@ -383,7 +381,7 @@ namespace pan
               Quark parent;
             };
 
-            typedef std::list<Added> added_t;
+            typedef std::map<Quark,Added> added_t;
             added_t added;
 
             /**
@@ -403,7 +401,7 @@ namespace pan
               Quark new_parent;
             };
 
-            typedef std::list<Reparent> reparented_t;
+            typedef std::map<Quark,Reparent> reparented_t;
             reparented_t reparented;
 
             quarks_t removed;
@@ -437,10 +435,12 @@ namespace pan
 
           /** the quirky way of incrementing 'it' is to prevent it from being
               invalidated if on_tree_change() calls remove_listener() */
-          void fire_diffs (const Diffs& diffs) const {
-            listeners_t::iterator it, e;
-            for (it=_listeners.begin(), e=_listeners.end(); it!=e; )
-              (*it++)->on_tree_change (diffs);
+          void fire_diffs (const Diffs& d) const {
+            if (!d.added.empty() || !d.removed.empty() || !d.reparented.empty() || !d.changed.empty()) {
+              listeners_t::iterator it, e;
+              for (it=_listeners.begin(), e=_listeners.end(); it!=e; )
+                (*it++)->on_tree_change (d);
+            }
           }
               
 
@@ -537,7 +537,7 @@ namespace pan
                                          const Quark          & group,
                                          const StringView     & subject,
                                          const StringView     & author,
-                                         const StringView     & time,
+                                         const time_t           time,
                                          const StringView     & message_id,
                                          const StringView     & references,
                                          const unsigned long    byte_count,
