@@ -25,8 +25,8 @@ extern "C" {
   #include <gtk/gtk.h>
 }
 #include <pan/general/debug.h>
-#include <pan/general/foreach.h>
 #include <pan/general/log.h>
+#include <pan/general/macros.h>
 #include <pan/general/quark.h>
 #include <pan/general/text-match.h>
 #include <pan/general/time-elapsed.h>
@@ -68,23 +68,20 @@ namespace
   std::string
   get_short_name (const StringView& in)
   {
-    static const StringView moderated ("moderated");
-    static const StringView d ("d");
+    StringView myline, long_token_one, long_token_two;
 
-    StringView myline, long_token;
-
-    // find the long token -- use the last, unless that's "moderated" or "d"
+	// pop off the last two tokens
     myline = in;
-    myline.pop_last_token (long_token, '.');
-    if (!myline.empty() && (long_token==moderated || long_token==d))
-      myline.pop_last_token (long_token, '.');
+    myline.pop_last_token (long_token_two, '.');
+	if (!myline.empty())
+	  myline.pop_last_token (long_token_one, '.');
 
-    // build a new string where each token is shortened except for long_token
+    // build a new string where each token is shortened except for the long_tokens
     std::string out;
     myline = in;
     StringView tok;
     while (myline.pop_token (tok, '.')) {
-      out.insert (out.end(), tok.begin(), (tok==long_token ? tok.end() : tok.begin()+1));
+      out.insert (out.end(), tok.begin(), ((tok==long_token_one || tok==long_token_two) ? tok.end() : tok.begin()+1));
       out += '.';
     }
     if (!out.empty())
@@ -149,7 +146,7 @@ GroupPane :: get_full_selection () const
 }
 
 void
-GroupPane ::  do_popup_menu (GtkWidget *treeview, GdkEventButton *event, gpointer pane_g)
+GroupPane ::  do_popup_menu (GtkWidget*, GdkEventButton *event, gpointer pane_g)
 {
   GroupPane * self (static_cast<GroupPane*>(pane_g));
   GtkWidget * menu (self->_action_manager.get_action_widget ("/group-pane-popup"));
@@ -169,9 +166,9 @@ namespace
 }
 
 void
-GroupPane :: on_row_activated (GtkTreeView          * treeview,
-                               GtkTreePath          * path,
-                               GtkTreeViewColumn    * col,
+GroupPane :: on_row_activated (GtkTreeView          * ,
+                               GtkTreePath          * ,
+                               GtkTreeViewColumn    * ,
                                gpointer               pane_g)
 {
   // user may have activated by selecting the row... wait for the GtkTreeSelection to catch up
@@ -182,7 +179,7 @@ namespace
 {
   bool row_collapsed_or_expanded (false);
 
-  void row_collapsed_or_expanded_cb (GtkTreeView *view, GtkTreeIter *iter, GtkTreePath *path, gpointer unused)
+  void row_collapsed_or_expanded_cb (GtkTreeView*, GtkTreeIter*, GtkTreePath*, gpointer)
   {
     row_collapsed_or_expanded = true;
   }
@@ -305,7 +302,7 @@ namespace
   }
 
   void
-  delete_rows (gpointer data, GObject * dead_object)
+  delete_rows (gpointer data, GObject*)
   {
     delete [] static_cast<MyRow*>(data);
   }
@@ -315,7 +312,7 @@ namespace
   struct NoopRowDispose: public PanTreeStore::RowDispose
   {
     virtual ~NoopRowDispose () { }
-    virtual void row_dispose (PanTreeStore *store, PanTreeStore::Row* row) { }
+    virtual void row_dispose (PanTreeStore*, PanTreeStore::Row*) { }
   };
 
   NoopRowDispose noop_row_dispose;
@@ -476,7 +473,7 @@ GroupPane :: on_group_read (const Quark& groupname)
        _dirty_groups_idle_tag = g_timeout_add (333, dirty_groups_idle, this);
 }
 void
-GroupPane :: on_group_counts (const Quark& groupname, unsigned long unread, unsigned long total)
+GroupPane :: on_group_counts (const Quark& groupname, unsigned long, unsigned long)
 {
   on_group_read (groupname);
 }
@@ -552,8 +549,8 @@ namespace
   }
 
   gboolean search_entry_focus_in_cb (GtkWidget     * w,
-                                     GdkEventFocus * unused1,
-                                     gpointer        unused2)
+                                     GdkEventFocus * ,
+                                     gpointer        )
   {
     gtk_widget_modify_text (w, GTK_STATE_NORMAL, NULL); // resets
     set_search_entry (w, search_text.c_str());
@@ -573,8 +570,8 @@ namespace
   }
 
   gboolean search_entry_focus_out_cb (GtkWidget     * w,
-                                      GdkEventFocus * unused1,
-                                      gpointer        unused2)
+                                      GdkEventFocus * ,
+                                      gpointer        )
   {
     refresh_search_entry (w);
     return false;
@@ -594,13 +591,13 @@ namespace
     }
   }
 
-  void search_entry_activated (GtkEntry * entry, gpointer pane_gpointer)
+  void search_entry_activated (GtkEntry*, gpointer pane_gpointer)
   {
     search_activate (static_cast<GroupPane*>(pane_gpointer));
     remove_activate_soon_tag ();
   }
 
-  void clear_button_clicked_cb (SexyIconEntry *e, SexyIconEntryPosition icon_pos, int button, gpointer pane_gpointer)
+  void clear_button_clicked_cb (SexyIconEntry *e, SexyIconEntryPosition, int, gpointer pane_gpointer)
   {
     set_search_entry (GTK_WIDGET(e), "");
     refresh_search_entry (GTK_WIDGET(e));
@@ -640,11 +637,11 @@ namespace
   bool shorten;
 
   void
-  render_group_name (GtkTreeViewColumn * col,
+  render_group_name (GtkTreeViewColumn * ,
                      GtkCellRenderer   * renderer,
                      GtkTreeModel      * model,
                      GtkTreeIter       * iter,
-                     gpointer            user_data)
+                     gpointer            )
   {
     PanTreeStore * tree (PAN_TREE_STORE(model));
     MyRow * row (dynamic_cast<MyRow*>(tree->get_row (iter)));
@@ -781,11 +778,11 @@ GroupPane :: read_next_group ()
 namespace
 {
   gboolean
-  select_func (GtkTreeSelection * selection,
+  select_func (GtkTreeSelection * ,
                GtkTreeModel     * model,
                GtkTreePath      * path,
-               gboolean           path_currently_selected,
-               gpointer           unused)
+               gboolean           ,
+               gpointer           )
   {
     GtkTreeIter iter;
     PanTreeStore * store (PAN_TREE_STORE(model));
@@ -817,11 +814,19 @@ GroupPane :: create_filter_entry ()
 }
 
 void
-GroupPane :: on_selection_changed (GtkTreeSelection * sel, gpointer pane_gpointer)
+GroupPane :: on_selection_changed (GtkTreeSelection*, gpointer pane_gpointer)
 {
   GroupPane * self (static_cast<GroupPane*>(pane_gpointer));
   Quark group (self->get_first_selection());
-  self->_action_manager.sensitize_action ("show-group-preferences-dialog", is_group(group));
+  const bool have_group = is_group (group);
+  static const char* actions_that_require_a_group[] = {
+    "show-group-preferences-dialog",
+    "subscribe", "unsubscribe",
+    "read-selected-group", "mark-groups-read", "delete-groups-articles",
+    "get-new-headers-in-selected-groups", "download-headers"
+  };
+  for (int i=0, n=G_N_ELEMENTS(actions_that_require_a_group); i<n; ++i)
+    self->_action_manager.sensitize_action (actions_that_require_a_group[i], have_group);
 }
 
 GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs):
@@ -842,6 +847,9 @@ GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs)
   g_object_unref (G_OBJECT(_tree_store)); // will die with the view
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW(_tree_view), false);
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(_tree_view), false);
+#if GTK_CHECK_VERSION(2,8,0)
+  gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW(_tree_view), true);
+#endif
 #if GTK_CHECK_VERSION(2,10,0)
   gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW(_tree_view), true);
 #endif
@@ -853,9 +861,6 @@ GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs)
   g_signal_connect (selection, "changed", G_CALLBACK(on_selection_changed), this);
   on_selection_changed (selection, this);
 
-#if GTK_CHECK_VERSION(2,8,0)
-  gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW(_tree_view), true);
-#endif
 
   g_signal_connect (_tree_view, "row_collapsed", G_CALLBACK(row_collapsed_or_expanded_cb), 0);
   g_signal_connect (_tree_view, "row_expanded", G_CALLBACK(row_collapsed_or_expanded_cb), 0);
@@ -917,14 +922,14 @@ GroupPane :: refresh_font ()
 }
 
 void
-GroupPane :: on_prefs_flag_changed (const StringView& key, bool value)
+GroupPane :: on_prefs_flag_changed (const StringView& key, bool)
 {
   if (key == "group-pane-font-enabled")
     refresh_font ();
 }
 
 void
-GroupPane :: on_prefs_string_changed (const StringView& key, const StringView& value)
+GroupPane :: on_prefs_string_changed (const StringView& key, const StringView&)
 {
   if (key == "group-pane-font")
     refresh_font ();
