@@ -27,8 +27,8 @@ extern "C" {
 }
 #include <pan/general/debug.h>
 #include <pan/general/file-util.h>
-#include <pan/general/foreach.h>
 #include <pan/general/log.h>
+#include <pan/general/macros.h>
 #include <pan/general/quark.h>
 #include <pan/general/string-view.h>
 #include <pan/general/utf8-utils.h>
@@ -40,7 +40,6 @@ using namespace pan;
 namespace
 {
   typedef std::vector<Task*> tasks_t;
-  typedef std::map<unsigned int,Article::Part> number_to_part_t;
 
   struct MyContext
   {
@@ -48,7 +47,7 @@ namespace
     std::string text;
     std::string path;
     Article a;
-    number_to_part_t parts;
+    PartBatch parts;
     tasks_t tasks;
     ArticleCache& cache;
     ArticleRead& read;
@@ -66,7 +65,6 @@ namespace
       groups.clear ();
       text.clear ();
       path.clear ();
-      parts.clear ();
       a.clear ();
       bytes = 0;
       number = 0;
@@ -74,12 +72,12 @@ namespace
   };
 
   // called for open tags <foo bar='baz'>
-  void start_element (GMarkupParseContext *context,
+  void start_element (GMarkupParseContext *context         UNUSED,
                       const gchar         *element_name,
                       const gchar        **attribute_names,
                       const gchar        **attribute_vals,
                       gpointer             user_data,
-                      GError             **error)
+                      GError             **error           UNUSED)
   {
     MyContext& mc (*static_cast<MyContext*>(user_data));
 
@@ -103,10 +101,10 @@ namespace
   }
 
   // Called for close tags </foo>
-  void end_element    (GMarkupParseContext *context,
+  void end_element    (GMarkupParseContext *context       UNUSED,
                        const gchar         *element_name,
                        gpointer             user_data,
-                       GError             **error)
+                       GError             **error         UNUSED)
   {
     MyContext& mc (*static_cast<MyContext*>(user_data));
 
@@ -115,22 +113,20 @@ namespace
 
     else if (!strcmp(element_name, "segment") && mc.number && !mc.text.empty()) {
       const std::string mid ("<" + mc.text + ">");
-      if (mc.a.message_id.empty())
-          mc.a.message_id = mid;
-      Article::Part& part (mc.parts[mc.number]);
-      part.bytes = mc.bytes;
-      part.set_message_id (mc.a.message_id, mid);
+      if (mc.a.message_id.empty()) {
+        mc.a.message_id = mid;
+        mc.parts.init (mid);
+      }
+      mc.parts.add_part (mc.number, mid, mc.bytes);
     }
 
     else if (!strcmp(element_name,"path"))
       mc.path = mc.text;
 
-    else if (!mc.parts.empty() && !strcmp (element_name, "file"))
+    else if (!strcmp (element_name, "file"))
     {
-      // populate mc.a.parts
-      mc.a.set_part_count (mc.parts.rbegin()->first);
-      foreach (number_to_part_t, mc.parts, it)
-        mc.a.get_part(it->first).swap (it->second);
+      mc.parts.sort ();
+      mc.a.set_parts (mc.parts);
 
       foreach_const (quarks_t, mc.groups, git) {
         quarks_t servers;
@@ -143,11 +139,11 @@ namespace
     }
   }
 
-  void text (GMarkupParseContext *context,
+  void text (GMarkupParseContext *context    UNUSED,
              const gchar         *text,
              gsize                text_len,
              gpointer             user_data,
-             GError             **error)
+             GError             **error      UNUSED)
   {
     static_cast<MyContext*>(user_data)->text.assign (text, text_len);
   }
@@ -265,17 +261,11 @@ NZB :: nzb_to_xml (std::ostream             & out,
 
     // now for the parts...
     out << indent(depth++) << "<segments>\n";
-    int part_number (0);
-    foreach_const (Article::parts_t, a.parts, pit)
+    for (Article::part_iterator it(a.pbegin()), end(a.pend()); it!=end; ++it)
     {
-      ++part_number;
-
-      // incomplete multipart...
-      if (pit->empty())
-        continue;
+      std::string mid = it.mid ();
 
       // remove the surrounding < > as per nzb spec
-      std::string mid (pit->get_message_id (a.message_id));
       if (mid.size()>=2 && mid[0]=='<') {
         mid.erase (0, 1);
         mid.resize (mid.size()-1);
@@ -283,8 +273,8 @@ NZB :: nzb_to_xml (std::ostream             & out,
 
       // serialize this part
       out << indent(depth)
-          << "<segment" << " bytes=\"" << pit->bytes << '"'
-                        << " number=\"" << part_number << '"'
+          << "<segment" << " bytes=\"" << it.bytes() << '"'
+                        << " number=\"" << it.number() << '"'
                         << ">";
       escaped(out, mid);
       out  << "</segment>\n";
