@@ -19,7 +19,7 @@
 
 #include <config.h>
 #include <pan/general/debug.h>
-#include <pan/general/foreach.h>
+#include <pan/general/macros.h>
 #include <pan/general/messages.h>
 #include <pan/data/server-info.h>
 #include "queue.h"
@@ -33,11 +33,15 @@ using namespace pan;
 
 Queue :: Queue (ServerInfo         & server_info,
                 TaskArchive        & archive,
-                Socket::Creator    * socket_creator, 
+                Socket::Creator    * socket_creator,
+                WorkerPool         & pool,
                 bool                 online):
   _server_info (server_info),
   _is_online (online),
   _socket_creator (socket_creator),
+  _worker_pool (pool),
+  _decoder (pool),
+  _decoder_task (0),
   _needs_saving (false),
   _last_time_saved (0),
   _archive (archive)
@@ -70,7 +74,7 @@ Queue :: on_pool_has_nntp_available (const Quark& server)
 }
 
 void
-Queue :: on_pool_error (const Quark& server, const StringView& message)
+Queue :: on_pool_error (const Quark& server UNUSED, const StringView& message)
 {
   fire_queue_error (message);
 }
@@ -112,6 +116,19 @@ Queue :: upkeep ()
     _last_time_saved = now;
   }
 
+  // do upkeep on the first queued task.
+  // the CPU goes crazy if we run upkeep on _all_ queued tasks,
+  // but we need to run upkeep on at least one queued task
+  // to wake up the queue if it's stuck from a bad connection.
+  // ref #352170, #354779
+  foreach_const (tasks_t, tmp, it) {
+    Task * task (*it);
+    if (task->get_state()._work == Task::NEED_NNTP) {
+      process_task (task);
+      break;
+    }
+  }
+
   // remove completed tasks.
   foreach_const (tasks_t, tmp, it) {
     Task * task  (*it);
@@ -122,7 +139,7 @@ Queue :: upkeep ()
 
   // upkeep on running tasks... this lets us pop open
   // extra connections if the task can handle >1 connection
-  std::set<Task*> active; 
+  std::set<Task*> active;
   foreach (nntp_to_task_t, _nntp_to_task, it)
     active.insert (it->second);
   foreach (std::set<Task*>, active, it)
@@ -170,10 +187,21 @@ Queue :: get_task_counts (int& active, int& total)
   std::set<Task*> active_tasks;
   foreach_const (nntp_to_task_t, _nntp_to_task, it)
     active_tasks.insert (it->second);
+  if (_decoder_task)
+    active_tasks.insert (_decoder_task);
   active = active_tasks.size ();
   total = _tasks.size ();
 }
 
+void
+Queue :: give_task_a_decoder (Task * task)
+{
+  const bool was_active (task_is_active (task));
+  _decoder_task = task;
+  if (!was_active)
+    fire_task_active_changed (task, true);
+  task->give_decoder (this, &_decoder); // it's active now...
+}
 
 void
 Queue :: give_task_a_connection (Task * task, NNTP * nntp)
@@ -186,7 +214,6 @@ Queue :: give_task_a_connection (Task * task, NNTP * nntp)
   task->give_nntp (this, nntp);
 }
 
-
 void
 Queue :: process_task (Task * task)
 {
@@ -209,9 +236,9 @@ Queue :: process_task (Task * task)
   else if (_stopped.count(task))
   {
     debug ("stopped");
-    // do nothing
+    task->stop();
   }
-  else if (state._health == FAIL)
+  else if ((state._health == ERR_COMMAND) || (state._health == ERR_LOCAL))
   {
     debug ("fail");
     // do nothing
@@ -221,12 +248,17 @@ Queue :: process_task (Task * task)
     debug ("working");
     // do nothing
   }
-  else while (state._work == Task::NEED_NNTP)
+  else if (state._work == Task::NEED_DECODER)
+  {
+    if (!_decoder_task)
+      give_task_a_decoder (task);
+  }
+  else while (_is_online && (state._work == Task::NEED_NNTP))
   {
     // make the requests...
     const Task::State::unique_servers_t& servers (state._servers);
     foreach_const (Task::State::unique_servers_t, servers, it)
-      get_pool(*it).request_nntp ();
+      get_pool(*it).request_nntp (_worker_pool);
 
     Quark server;
     if (!find_best_server (servers, server))
@@ -244,26 +276,34 @@ Queue :: process_task (Task * task)
 ****
 ***/
 
+Task*
+Queue :: find_first_task_needing_decoder ()
+{
+  foreach (TaskSet, _tasks, it) {
+    const Task::State& state ((*it)->get_state ());
+    if  ((state._work == Task::NEED_DECODER)
+      && (!_stopped.count (*it))
+      && (!_removing.count (*it)))
+      return *it;
+  }
+
+  return 0;
+}
+
 Task*
 Queue :: find_first_task_needing_server (const Quark& server)
 {
-  Task * retval (0);
-
-  foreach (TaskSet, _tasks, it)
-  {
+  foreach (TaskSet, _tasks, it) {
     const Task::State& state ((*it)->get_state ());
-    if  ((state._health != FAIL)
+    if  (((state._health != ERR_COMMAND) && (state._health != ERR_LOCAL))
       && (state._work == Task::NEED_NNTP)
       && (state._servers.count(server))
       && (!_stopped.count (*it))
       && (!_removing.count (*it)))
-    {
-      retval = *it;
-      break;
-    }
+      return *it;
   }
 
-  return retval;
+  return 0;
 }
 
 bool
@@ -302,21 +342,21 @@ Queue :: find_best_server (const Task::State::unique_servers_t& servers, Quark&
 ***/
 
 void
-Queue :: on_set_items_added (TaskSet& container, TaskSet::items_t& tasks, int pos)
+Queue :: on_set_items_added (TaskSet& container UNUSED, TaskSet::items_t& tasks, int pos)
 {
   _needs_saving = true;
   fire_tasks_added (pos, tasks.size());
 }
 
 void
-Queue :: on_set_item_removed (TaskSet& container, Task*& task, int pos)
+Queue :: on_set_item_removed (TaskSet& container UNUSED, Task*& task, int pos)
 {
   _needs_saving = true;
   fire_task_removed (task, pos);
 }
 
 void
-Queue :: on_set_item_moved (TaskSet& container, Task*& task, int new_pos, int old_pos)
+Queue :: on_set_item_moved (TaskSet& container UNUSED, Task*& task, int new_pos, int old_pos)
 {
   _needs_saving = true;
   fire_task_moved (task, new_pos, old_pos);
@@ -384,6 +424,9 @@ Queue :: set_online (bool online)
 {
   _is_online = online;
   fire_online_changed (_is_online);
+
+  if (_is_online)
+    upkeep ();
 }
 
 void
@@ -391,8 +434,10 @@ Queue :: stop_tasks (const tasks_t& tasks)
 {
   foreach_const (tasks_t, tasks, it) {
     Task * task (*it);
-    if (_tasks.index_of (task) != -1)
+    if (_tasks.index_of (task) != -1) {
       _stopped.insert (task);
+      process_task (task);
+    }
   }
 }
 
@@ -436,6 +481,9 @@ Queue :: add_tasks (const tasks_t& tasks, AddMode mode)
   tasks_t tmp (tasks);
   foreach (tasks_t, tmp, it)
     process_task (*it);
+
+  // maybe fire counts changed events...
+  fire_if_counts_have_changed ();
 }
 
 bool
@@ -454,6 +502,9 @@ Queue :: remove_latest_task ()
 bool
 Queue :: task_is_active (const Task * task) const
 {
+  if (task && task==_decoder_task)
+    return true;
+
   bool task_has_nntp (false);
   foreach_const (nntp_to_task_t, _nntp_to_task, it)
     if ((task_has_nntp = task==it->second))
@@ -474,9 +525,10 @@ Queue :: remove_task (Task * task)
   const int index (_tasks.index_of (task));
   pan_return_if_fail (index != -1);
 
-  if (task_is_active (task)) // wait for the NNTPs to finish
+  if (task_is_active (task)) // wait for the Task to finish
   {
-    debug ("can't delete this task right now because it's got server connections");
+    debug ("can't delete this task right now because it's active");
+    task->stop();
     _removing.insert (task);
   }
   else // no NNTPs working, we can remove right now.
@@ -490,13 +542,27 @@ Queue :: remove_task (Task * task)
     _tasks.remove (index);
     delete task;
   }
+
+  // maybe fire counts changed events...
+  fire_if_counts_have_changed ();
 }
 
 void
 Queue :: get_all_task_states (task_states_t& setme)
 {
-  setme.tasks.clear ();
-  setme.tasks.insert (setme.tasks.begin(), _tasks.begin(), _tasks.end());
+  setme.tasks.clear();
+  setme.tasks.reserve(_tasks.size());
+
+  std::vector<Task *> & need_decode = setme._need_decode.get_container();
+  need_decode.clear();
+  need_decode.reserve(setme.tasks.capacity());
+
+  foreach(TaskSet, _tasks, it) {
+    setme.tasks.push_back(*it);
+    if ((*it)->get_state()._work == Task::NEED_DECODER)
+      need_decode.push_back(*it);
+  }
+  setme._need_decode.sort();
 
   setme._queued.get_container() = setme.tasks;
   setme._queued.sort ();
@@ -510,14 +576,16 @@ Queue :: get_all_task_states (task_states_t& setme)
   removing.insert (removing.end(), _removing.begin(), _removing.end());
 
   std::vector<Task*>& running (setme._running.get_container());
-  std::set<Task*> tmp; 
+  std::set<Task*> tmp;
   foreach (nntp_to_task_t, _nntp_to_task, it) tmp.insert (it->second);
   running.clear ();
   running.insert (running.end(), tmp.begin(), tmp.end());
+
+  setme._decoding = _decoder_task;
 }
 
 void
-Queue :: check_in (NNTP * nntp, bool is_ok)
+Queue :: check_in (NNTP * nntp, Health nntp_health)
 {
   Task * task (_nntp_to_task[nntp]);
 
@@ -526,9 +594,10 @@ Queue :: check_in (NNTP * nntp, bool is_ok)
   // returning the NNTP to the pool and checking it out again.
   const Task::State state (task->get_state ());
 
-  if (is_ok
-    && (state._health!=FAIL)
-    && (state._work==Task::NEED_NNTP)
+  if ((nntp_health != ERR_NETWORK)
+    && _is_online
+    && ((state._health != ERR_COMMAND) && (state._health != ERR_LOCAL))
+    && (state._work == Task::NEED_NNTP)
     && !_removing.count(task)
     && state._servers.count(nntp->_server)
     && find_first_task_needing_server(nntp->_server)==task)
@@ -541,21 +610,48 @@ Queue :: check_in (NNTP * nntp, bool is_ok)
     Task * task (_nntp_to_task[nntp]);
     _nntp_to_task.erase (nntp);
 
-    // take care of the task's state
-    const bool is_active (task_is_active (task));
-    if (!is_active) // if it's not active anymore...
-      fire_task_active_changed (task, is_active);
+    // notify the listeners if the task isn't active anymore...
+    if (!task_is_active (task))
+      fire_task_active_changed (task, false);
+
+    // if we encountered a local error, fire an error message.
+    if (state._health == ERR_LOCAL)
+      fire_queue_error ("");
 
     // return the nntp to the pool
     const Quark& servername (nntp->_server);
     NNTP_Pool& pool (get_pool (servername));
-    pool.check_in (nntp, is_ok);
+    pool.check_in (nntp, nntp_health);
 
     // what to do now with this task...
     process_task (task);
   }
 }
 
+void
+Queue :: check_in (Decoder* decoder UNUSED, Task* task)
+{
+  // take care of our decoder counting...
+  _decoder_task = 0;
+
+  // notify the listeners if the task isn't active anymore...
+  if (!task_is_active (task))
+    fire_task_active_changed (task, false);
+
+  // if the task hit an error, fire an error message
+  const Task::State state (task->get_state ());
+  if (state._health == ERR_LOCAL)
+    fire_queue_error ("");
+
+  // pass our worker thread on to another task
+  Task * next = find_first_task_needing_decoder ();
+  if (next && (next!=task))
+    process_task (next);
+
+  // what to do now with this task...
+  process_task (task);
+}
+
 void
 Queue :: move_up (const tasks_t& tasks)
 {
@@ -656,3 +752,48 @@ Queue :: get_task_speed_KiBps (const Task  * task,
   setme_KiBps = KiBps;
   setme_connections = connections;
 }
+
+void
+Queue :: get_stats (unsigned long   & queued_count,
+                    unsigned long   & running_count,
+                    unsigned long   & stopped_count,
+                    uint64_t        & KiB_remain,
+                    double          & KiBps,
+                    int             & hours_remain,
+                    int             & minutes_remain,
+                    int             & seconds_remain)
+{
+  KiB_remain = 0;
+  KiBps = 0;
+  queued_count = running_count = stopped_count = 0;
+  hours_remain = minutes_remain = seconds_remain = 0;
+
+  task_states_t tasks;
+  get_all_task_states (tasks);
+  foreach_const (tasks_t, tasks.tasks, it)
+  {
+    Task * task (*it);
+
+    const Queue::TaskState state (tasks.get_state (task));
+    if (state == Queue::RUNNING || state == Queue::DECODING)
+      ++running_count;
+    else if (state == Queue::STOPPED)
+      ++stopped_count;
+    else if (state == Queue::QUEUED || state == Queue::QUEUED_FOR_DECODE)
+      ++queued_count;
+
+    if (state==Queue::RUNNING || state==Queue::QUEUED)
+      KiB_remain += task->get_bytes_remaining ();
+  }
+
+  const unsigned long task_count (running_count + queued_count);
+  KiBps = get_speed_KiBps ();
+  if (task_count) {
+    const double KiB ((double)KiB_remain / 1024);
+    unsigned long tmp (KiBps>0.01 ? ((unsigned long)(KiB / KiBps)) : 0);
+    seconds_remain = tmp % 60ul; tmp /= 60ul;
+    minutes_remain = tmp % 60ul; tmp /= 60ul;
+    hours_remain   = tmp;
+  }
+}
+
