@@ -40,7 +40,7 @@ extern "C"
 }
 
 #include <pan/general/debug.h>
-#include <pan/general/foreach.h>
+#include <pan/general/macros.h>
 #include <pan/general/log.h>
 #include <pan/general/string-view.h>
 #include "gnksa.h"
